/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.wsas.admin.service;

import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ServiceContext;
import org.apache.axis2.engine.AxisConfiguration;
import org.wso2.utils.i18n.Messages;


/**
 * Parent class for all admin services
 *
 * @see org.wso2.wsas.admin.service.GlobalAdmin
 * @see org.wso2.wsas.admin.service.ModuleAdmin
 * @see org.wso2.wsas.admin.service.OperationAdmin
 * @see org.wso2.wsas.admin.service.ServerAdmin
 * @see org.wso2.wsas.admin.service.ServiceAdmin
 * @see org.wso2.wsas.admin.service.ServiceGroupAdmin
 * @see org.wso2.wsas.admin.service.UserAdmin
 */
public abstract class AbstractAdmin {

    protected AxisConfiguration axisConfig;
    protected ConfigurationContext configCtx;

    //this method will be using dependency injection
    public void init(ServiceContext serviceContext) {
        this.setConfigContext(serviceContext.getConfigurationContext());
        this.setAxisConfig(serviceContext.getConfigurationContext()
                .getAxisConfiguration());
    }


    public AxisConfiguration getAxisConfig() {
        if (axisConfig == null) {
            throw new RuntimeException(Messages.getMessage("InvalidAxisConfiguration"));
        }
        return axisConfig;
    }

    public void setAxisConfig(AxisConfiguration axisConfig) {
        this.axisConfig = axisConfig;
    }

    public void setConfigContext(ConfigurationContext configurationContext) {
        this.configCtx = configurationContext;
    }

    public ConfigurationContext getConfigContext() {
        return configCtx;
    }
}
