/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.wsas.admin.service;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMAttribute;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axiom.om.util.StAXUtils;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.AxisService;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.utils.ServerConfiguration;
import org.wso2.ws.dataservice.DBConstants;
import org.wso2.wsas.ServerConstants;
import org.wso2.wsas.util.WsasUtils;
import org.wso2.wsas.admin.service.util.DBServerData;

import au.com.bytecode.opencsv.CSVReader;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Admin operations for DataService
 */
public class DataServiceAdmin extends AbstractAdmin {
	
	private static final Log log = LogFactory.getLog(DataServiceAdmin.class);
    public DataServiceAdmin() {
    }

    public DataServiceAdmin(ConfigurationContext configCtx) {
        this.configCtx = configCtx;
        this.axisConfig = configCtx.getAxisConfiguration();
    }

    public OMElement getDataServiceContents(String serviceId) throws AxisFault {
        AxisService axisService = getAxisConfig().getService(serviceId);
        String fileParth =
                (String) axisService.getParameter(DBConstants.DB_SERVICE_CONFIG_FILE).getValue();
        StringBuffer fileContents = new StringBuffer();
        try {
            BufferedReader in = new BufferedReader(new FileReader(fileParth));
            String str;
            while ((str = in.readLine()) != null) {
                fileContents.append(str);
            }
            in.close();
        } catch (IOException e) {
            throw new AxisFault(
                    "Error while reading the contents from the service config file for service "
                            + serviceId, e);
        }

        OMElement returnEle;
        try {
            XMLStreamReader xmlSR =
                    StAXUtils.createXMLStreamReader(new ByteArrayInputStream(fileContents.toString().getBytes()));

            returnEle = new StAXOMBuilder(xmlSR).getDocumentElement();
            // new BufferedReader(new InputStreamReader(
        } catch (Exception e) {
            throw new AxisFault(
                    "Error while converting the contents of the configuration file into and OMElement for service "
                            + serviceId, e);
        }
        return returnEle;
    }

    public void saveDataServiceContents(String serviceId, String configContents) throws AxisFault {
        OMElement configEle;
        try {
            XMLStreamReader xmlSR =
                    StAXUtils.createXMLStreamReader(new ByteArrayInputStream(configContents
                            .getBytes()));

            configEle = new StAXOMBuilder(xmlSR).getDocumentElement();
            //TODO : change method signature 
            OMAttribute attribute = configEle.getAttribute(new QName("name"));
            serviceId = attribute.getAttributeValue();
            // new BufferedReader(new InputStreamReader(
        } catch (Exception e) {
            throw new AxisFault("Error while converting the contents echoed to an OMElement", e);
        }

        Iterator queries = configEle.getChildrenWithName(new QName("query"));

        while (queries.hasNext()) {
            OMElement query = (OMElement) queries.next();
            Iterator results = query.getChildrenWithName(new QName("result"));

            while (results.hasNext()) {
                OMElement result = (OMElement) results.next();
                // Only selected if the all simple version is returned from the
                // client.
                OMAttribute rowNameAttribute = result.getAttribute(new QName("rowname"));
                if (rowNameAttribute != null) {
                    String attValue = rowNameAttribute.getAttributeValue();
                    result.removeAttribute(rowNameAttribute);
                    result.addAttribute(OMAbstractFactory.getOMFactory().createOMAttribute(
                            "rowName", OMAbstractFactory.getOMFactory().createOMNamespace("", ""),
                            attValue));
                }
            }

            Iterator params = query.getChildrenWithName(new QName("param"));

            while (params.hasNext()) {
                OMElement param = (OMElement) params.next();
                // Only selected if the all simple version is returned from the
                // client.
                OMAttribute sqlTypeAttribute = param.getAttribute(new QName("sqltype"));
                if (sqlTypeAttribute != null) {
                    String attValue = sqlTypeAttribute.getAttributeValue();
                    param.removeAttribute(sqlTypeAttribute);
                    param.addAttribute(OMAbstractFactory.getOMFactory().createOMAttribute(
                            "sqlType", OMAbstractFactory.getOMFactory().createOMNamespace("", ""),
                            attValue));
                }
            }
        }

        String dataServiceFilePath;
        AxisService axisService = getAxisConfig().getService(serviceId);
        if (axisService == null) {
            // New service
            String axis2RepoDirectory =
                    this.configCtx.getAxisConfiguration().getRepository().getPath();
            String repoDirectory = (String) this.configCtx.getProperty(DBConstants.DB_SERVICE_REPO);
            String fileExtension =
                    (String) this.configCtx.getProperty(DBConstants.DB_SERVICE_EXTENSION);
            
            String dataServiceDirectory = axis2RepoDirectory + File.separator + repoDirectory;
            dataServiceFilePath = dataServiceDirectory + File.separator
                            + serviceId + "." + fileExtension;

            //create the directory, if it does not exist
            File directory = new File(dataServiceDirectory);
            directory.mkdirs();
            
            File file = new File(dataServiceFilePath);
            try {
                file.createNewFile();
            } catch (IOException e) {
                throw new AxisFault(
                        "Error while creating the file for the new service config file for the new service "
                                + serviceId, e);
            }

        } else {
        	dataServiceFilePath =
                    (String) axisService.getParameter(DBConstants.DB_SERVICE_CONFIG_FILE)
                            .getValue();
        }

        try {
            BufferedWriter out = new BufferedWriter(new FileWriter(dataServiceFilePath));
            configEle.serialize(out);
            // out.write(configContents);
            out.close();
        } catch (IOException e) {
            throw new AxisFault(
                    "Error while writing the contents for the service config file for the new service "
                            + serviceId, e);
        } catch (XMLStreamException e) {

            throw new AxisFault("Error while serializing the config OMElement for the service "
                    + serviceId, e);

        }

    }

    public OMElement echoDataElement(String dataElement) throws AxisFault {

        OMElement returnEle;
        String axis2Repo = ServerConfiguration.getInstance().
                getFirstProperty(ServerConfiguration.AXIS2_CONFIG_REPO_LOCATION);
        if (WsasUtils.isURL(axis2Repo)) {
            throw new AxisFault("WSAS is running from a URL repository " + axis2Repo +
                                ". Cannot create Data Services for URL repositories");
        } else {
            try {
                XMLStreamReader xmlSR =
                        StAXUtils
                                .createXMLStreamReader(new ByteArrayInputStream(dataElement.getBytes()));

                returnEle = new StAXOMBuilder(xmlSR).getDocumentElement();
                // new BufferedReader(new InputStreamReader(
            } catch (Exception e) {
                throw new AxisFault("Error while converting the contents echoed to an OMElement", e);
            }
        }
        return returnEle;
    }

    /*
     * Returns list of known database server jdbc urls and driver class names from server.xml
     */
    public DBServerData[] getDatabaseUrlDriverList() {
        ArrayList databaseServers = new ArrayList();
        DBServerData databaseServerData;
        ServerConfiguration serverConfiguration = ServerConfiguration.getInstance();
        OMElement documentElement = serverConfiguration.getDocumentElement();
        OMElement databaseServersElement =
                documentElement.getFirstChildWithName(
                        new QName(ServerConstants.WSO2WSAS_XML_NAMESPACE, "DatabaseServers"));

        Iterator serverElementIterator = databaseServersElement.getChildrenWithName(
        		new QName(ServerConstants.WSO2WSAS_XML_NAMESPACE, "Server"));

        for (; serverElementIterator.hasNext();) {
            OMElement serverElement = (OMElement) serverElementIterator.next();
            OMElement serverIdElement = serverElement.getFirstChildWithName(
                    new QName(ServerConstants.WSO2WSAS_XML_NAMESPACE, "Id"));
            OMElement jdbcUrlElement = serverElement.getFirstChildWithName(
                    new QName(ServerConstants.WSO2WSAS_XML_NAMESPACE, "JdbcURL"));
            OMElement driverClassElement = serverElement.getFirstChildWithName(
                    new QName(ServerConstants.WSO2WSAS_XML_NAMESPACE, "DriverClass"));
            
            databaseServerData = new DBServerData();
            databaseServerData.setServerId(serverIdElement.getText());
            databaseServerData.setJdbcUrl(jdbcUrlElement.getText());
            databaseServerData.setDriverClass(driverClassElement.getText());
            databaseServers.add(databaseServerData);
        }
        DBServerData[] databaseServerList = new DBServerData[databaseServers.size()];
        databaseServers.toArray(databaseServerList);
        return databaseServerList;
    }
    
    
    /*
     * loads first line of CSV file as the column header & returns an String array of column names
     * TODO : implement for Excel 
     */
    
    public String[] getHeaderColumnNames(String resourcePath,String hasHeaders,String dataSourceType){
    	log.debug("getHeaderColumnNames() called with params :"
    			+resourcePath+ " : "+hasHeaders+" : "+dataSourceType); 
    	CSVReader reader = null;
    	String[] columnHeader = null;
    	if(DBConstants.Query.CSV.equals(dataSourceType)){
        	InputStreamReader inputStreamReader = null;
        	try {
        		if (resourcePath.startsWith("http://")) {
        			URL url = new URL(resourcePath);				
        			inputStreamReader = new InputStreamReader(url.openStream());
        			reader = new CSVReader(inputStreamReader);
        		} else {
        			reader = new CSVReader(new FileReader(resourcePath));
        		}
        		
        		if(reader != null){
        			//read the first line, which contains column header
        			String[] firstLine = reader.readNext();
        			if(Boolean.valueOf(hasHeaders).booleanValue()){
        				//if hasHeader is set to true, first line contains column names
        				columnHeader = firstLine;
        			}else{
        				//create a dummy column header with number of columns
        				columnHeader = new String[firstLine.length]; 
        				for (int a = 0; a < firstLine.length; a++) {
        					columnHeader[a] = String.valueOf(a+1);
    					}
        			}
        		}
        	} catch (MalformedURLException e) {
        		log.error("Incorrect "+ dataSourceType +" URL : "+resourcePath, e);
        	} catch (IOException e) {
        		log.error("Error reading file : "+resourcePath, e);
        		e.printStackTrace();
        	}
    	}
        return columnHeader;
    }
}
