/*
* Copyright 2005,2006 WSO2, Inc. http://wso2.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*
*/
package org.wso2.throttle.module;


import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.*;
import org.apache.axis2.modules.Module;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.neethi.Assertion;
import org.apache.neethi.Policy;
import org.apache.neethi.PolicyEngine;
import org.wso2.throttle.Throttle;
import org.wso2.throttle.ThrottleConstants;
import org.wso2.throttle.ThrottleException;
import org.wso2.throttle.ThrottlePolicyProcessor;

import java.io.InputStream;

public class ThrottleModule implements Module {

    private static Log log = LogFactory.getLog(ThrottleModule.class.getName());

    /**
     * The QName of the module
     */
    private String name;

    private Policy defaultPolicy = null;
    private Throttle defaultThrottle = null;
    /**
     * initialize the module
     */
    public void init(ConfigurationContext configContext, AxisModule module) throws AxisFault {
        initDefaultPolicy();
        initDefaultThrottle();
        Throttle throttle;
        this.name = module.getName();
        PolicyInclude policyInclude = module.getPolicyInclude();
        if (policyInclude != null) {
            Policy policy = policyInclude.getEffectivePolicy();
            if (policy != null) {
                try {
                    throttle = ThrottlePolicyProcessor.processPolicy(policy);
                }
                catch (ThrottleException e) {

                    log.error("Error was ocuured when initing throttle module " + e.getMessage());
                    log.info("Throttling will occur using default module policy");

                    String id = policy.getId();
                    policyInclude.removePolicyElement(id);
                    defaultPolicy.setId(id);
                    policyInclude.addPolicyElement(PolicyInclude.AXIS_MODULE_POLICY,defaultPolicy);
                    throttle = defaultThrottle;
                }
                if (throttle != null) {
                    Parameter globalParameter = new Parameter();
                    globalParameter.setName(ThrottleConstants.GLOBAL_IP_BASED_THROTTLE_KEY);
                    globalParameter.setValue(throttle);
                    configContext.getAxisConfiguration().addParameter(globalParameter);
                }
            }
        }

    }

    public void engageNotify(AxisDescription axisDescription) throws AxisFault {

        String currentServiceName;
        if (axisDescription instanceof AxisService) {
            Throttle throttle;
            AxisService currentService = ((AxisService) axisDescription);
            AxisModule module = currentService.getAxisConfiguration().getModule(this.name);
            PolicyInclude policyInclude = currentService.getPolicyInclude();
            if (policyInclude != null) {
                try {
                    Policy currentPolicy = policyInclude.getPolicy();
                    if (currentPolicy == null) {
                        currentPolicy = policyInclude.getEffectivePolicy();
                        if (currentPolicy == null) {
                            policyInclude = module.getPolicyInclude();
                            if (policyInclude != null) {
                                currentPolicy = policyInclude.getEffectivePolicy();
                            }
                        }
                    }
                    throttle = ThrottlePolicyProcessor.processPolicy(currentPolicy);

                }
                catch (ThrottleException e) {

                    log.error("Error was ocuured when engaging throttle module for the service :" +
                        currentService.getName() + e.getMessage());
                    log.info("Throttling will occur using default module policy");
                    throttle = defaultThrottle;
                }
                if (throttle != null) {
                    Parameter service_throttle_Parameter = new Parameter();
                    service_throttle_Parameter.setName(currentService.getName());
                    service_throttle_Parameter.setValue(throttle);
                    axisDescription.getAxisConfiguration().addParameter(service_throttle_Parameter);
                }
            }
        } else if (axisDescription instanceof AxisOperation) {

            Throttle throttle;
            AxisOperation currentOperation = ((AxisOperation) axisDescription);
            currentServiceName = ((AxisService) currentOperation.getParent()).getName();
            AxisModule module = currentOperation.getAxisConfiguration().getModule(this.name);
            PolicyInclude policyInclude = currentOperation.getPolicyInclude();
            if (policyInclude != null) {
                try {
                    Policy currentPolicy = policyInclude.getPolicy();
                    if (currentPolicy == null) {
                        currentPolicy = policyInclude.getEffectivePolicy();
                        if (currentPolicy == null) {
                            policyInclude = module.getPolicyInclude();
                            if (policyInclude != null) {
                                currentPolicy = policyInclude.getEffectivePolicy();
                            }
                        }
                    }
                    throttle = ThrottlePolicyProcessor.processPolicy(currentPolicy);
                }

                catch (ThrottleException e) {
                    log.error("Error was ocuured when engaging throttle module for operation : " +
                        currentOperation.getName() + " in the service :" + currentServiceName + e.getMessage());
                    log.info("Throttling will occur using default module policy");
                    throttle = defaultThrottle;
                }

                if (throttle != null) {
                    Parameter operation_throttle_Parameter = new Parameter();
                    operation_throttle_Parameter.setName(currentServiceName + currentOperation.getName());
                    operation_throttle_Parameter.setValue(throttle);
                    axisDescription.getAxisConfiguration().addParameter(operation_throttle_Parameter);
                }

            }
        }
    }

    public void shutdown(ConfigurationContext configurationContext) throws AxisFault {
        //Todo
    }

    public void applyPolicy(Policy policy, AxisDescription axisDescription) throws AxisFault {

        // TODO
    }

    public boolean canSupportAssertion(Assertion assertion) {
        // TODO
        return true;
    }

    private void initDefaultPolicy() throws AxisFault {
        InputStream inputStream = this.getClass().getResourceAsStream("/resources/policy/default_module_policy.xml");
        if (inputStream != null) {
            defaultPolicy = PolicyEngine.getPolicy(inputStream);
        }
    }

    private void initDefaultThrottle() throws AxisFault {
        InputStream inputStream = this.getClass().getResourceAsStream("/resources/policy/default_module_policy.xml");
        if (inputStream != null) {
            try {

                defaultThrottle = ThrottlePolicyProcessor.processPolicy(defaultPolicy);
            } catch (ThrottleException e) {
                String msg = "Error during processing default throttle policy + system will not works" + e.getMessage();
                log.error(msg);
                throw new AxisFault(msg);
            }
        }
    }


}