/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package org.apache.woden.wsdl20;

import javax.xml.namespace.QName;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.woden.internal.wsdl20.DescriptionImpl;
import org.apache.woden.internal.wsdl20.ElementDeclarationImpl;
import org.apache.woden.types.NCName;
import org.apache.woden.wsdl20.xml.DescriptionElement;
import org.apache.woden.wsdl20.xml.InterfaceElement;
import org.apache.woden.wsdl20.xml.InterfaceFaultElement;

/**
 * Unit tests for the InterfaceFault class.
 * 
 * @author Graham Turrell (gturrell@apache.org)
 */
public class InterfaceFaultTest extends TestCase {

	private InterfaceFaultElement fFaultElement = null;
	private InterfaceFault fFault = null;
	private DescriptionElement fDescriptionElement = null;
	private Description fDescription = null;
	private InterfaceElement fInterfaceElement = null;
	private final String FAULT_NAME = "faultName";
	private final String INTF_NAME = "interfaceName";

	public static Test suite()
	{
	   return new TestSuite(InterfaceFaultTest.class);
	}
	
	/*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception 
    {
        super.setUp();
        fDescriptionElement = new DescriptionImpl();
        fInterfaceElement = fDescriptionElement.addInterfaceElement();
        fInterfaceElement.setName(new NCName(INTF_NAME));
        fFaultElement = fInterfaceElement.addInterfaceFaultElement();
        fFaultElement.setName(new NCName(FAULT_NAME));
        fFaultElement.setElementName(new QName(FAULT_NAME));
		fDescription = fDescriptionElement.toComponent();
		Interface iface = fDescription.getInterface(new QName(INTF_NAME));
		fFault = iface.getInterfaceFault(new QName(FAULT_NAME));
    }

    /*
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception 
    {
        super.tearDown();
    }
	
	/*
	 * Test that an (optional) ElementDecalaration QName can be successfully retrieved
	 */
	public void testGetElementDeclaration()
	{
		assertNotNull(fFault);
		
		// Default case
		assertNull("The retrieved Element Declaration when unset should be null", fFault.getElementDeclaration());
		
        // set and asscoiciate the Element Declaration
		ElementDeclarationImpl ed = new ElementDeclarationImpl();
        ed.setName(new QName(FAULT_NAME));
        ((DescriptionImpl)fDescriptionElement).addElementDeclaration(ed);
        fDescription = fDescriptionElement.toComponent();
        fFault = fDescription.getInterface(new QName(INTF_NAME)).getInterfaceFault(new QName(FAULT_NAME));
		assertEquals("The retrieved Element Declaration is not that which was set", 
				ed, fFault.getElementDeclaration());
	}

	/*
	 * Test that a (Mandatory) Name QName can be successfully retrieved
	 */
	public void testGetName()
	{
		QName faultName = new QName(FAULT_NAME);
		NCName faultNCName = new NCName(FAULT_NAME);
		fFaultElement.setName(faultNCName);
		fDescription = fDescriptionElement.toComponent();
		fFault = fDescription.getInterface(new QName(INTF_NAME)).getInterfaceFault(new QName(FAULT_NAME));
		assertEquals("The retrieved Interface Fault name is not that which was set", 
				faultName, fFault.getName());
	}
	
	/*
     * toElement()
     */
	public void testToElement() 
	{	
		assertEquals(fFaultElement, fFault.toElement());
	}

}
