/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package org.apache.woden.wsdl20;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.woden.internal.wsdl20.InterfaceMessageReferenceImpl;
import org.apache.woden.types.NCName;
import org.apache.woden.wsdl20.enumeration.Direction;
import org.apache.woden.wsdl20.xml.InterfaceMessageReferenceElement;



/**
 * Unit tests for the InterfaceMessageReference class.
 * 
 * @author Graham Turrell (gturrell@apache.org)
 */
public class InterfaceMessageReferenceTest extends TestCase {

	private InterfaceMessageReferenceElement fMessageReferenceElement = null;
	private InterfaceMessageReference fMessageReferenceComp = null;
	private final String CONTENT_MODEL = "contentModel";
	
	public static Test suite()
	{
	   return new TestSuite(InterfaceMessageReferenceTest.class);	   
	}
	   /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception 
    {
    	super.setUp();
    	fMessageReferenceElement = new InterfaceMessageReferenceImpl();
		fMessageReferenceElement.setDirection(Direction.OUT);
		fMessageReferenceElement.setMessageContentModel(CONTENT_MODEL);
    	fMessageReferenceComp = (InterfaceMessageReference) fMessageReferenceElement;  	
    }
    
    /*
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception 
    {
        super.tearDown();
    }
	
	/*
	 * Test that a (mandatory) direction can be successfully retrieved
	 */
	public void testGetDirection()
	{
		assertEquals("The retrieved FaultReference direction is not that which was set", 
				Direction.OUT, fMessageReferenceElement.getDirection());
	}

	/*
	 * Test that the (Mandatory) message label attribute ("messageLabel") can be successfully retrieved
	 */
	public void testGetMessageLabel()
	{
		NCName messageRefNCName = new NCName("messageRefName");
		fMessageReferenceElement.setMessageLabel(messageRefNCName);
		assertEquals("The retrieved Element name is not that which was set", 
				messageRefNCName, fMessageReferenceComp.getMessageLabel());
	}

	/* 
	 * Test that the (Mandatory) Message Content Model property can be successfully retrieved 
	 */
	public void testGetMessageContentModel()
	{
		assertEquals("The retrieved Message Content Model is not that which was set", 
				CONTENT_MODEL, fMessageReferenceComp.getMessageContentModel());
	}
	
	/*
     * toElement() - get element decalaration
     */
	public void testToElement() 
	{	
		assertEquals(fMessageReferenceElement, fMessageReferenceComp.toElement());
	}
}
