/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package org.apache.woden.wsdl20.xml;

import javax.xml.namespace.QName;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.woden.internal.wsdl20.InterfaceFaultImpl;
import org.apache.woden.types.NCName;

/**
 * Unit tests for the InterfaceFaultElement class.
 * 
 * @author Graham Turrell (gturrell@apache.org)
 */
public class InterfaceFaultElementTest extends TestCase {

	private InterfaceFaultElement fFault = new InterfaceFaultImpl();

	public static Test suite()
	{
	   return new TestSuite(InterfaceFaultElementTest.class);
	}
	
	/*
	 * Test that an (optional) ElementName QName can be successfully set and retrieved
	 */
	public void testSetGetElementName()
	{
		// Default case
		assertNull("The retrieved Element name when unset should be null", fFault.getElementName());
		
		QName elementName = new QName("elementName");
		fFault.setElementName(elementName);
		assertEquals("The retrieved Element name is not that which was set", 
				elementName, fFault.getElementName());
	}

	/*
	 * Test that a (Mandatory) Name QName can be successfully set and retrieved
	 */
	public void testSetGetName()
	{
		QName faultName = new QName("faultName");
		NCName faultNCName = new NCName("faultName");
		fFault.setName(faultNCName);
		assertEquals("The retrieved Element name is not that which was set", 
				faultName, fFault.getName());
	}

}
